# encoding: utf-8
import os
import shutil
import sys
import pytest
from os import path as op

from send2trash import send2trash as s2t

# import the two versions as well as the "automatic" version
if sys.platform == "win32":
    from send2trash.win.modern import send2trash as s2t_modern
    from send2trash.win.legacy import send2trash as s2t_legacy
else:
    pytest.skip("Skipping windows-only tests", allow_module_level=True)


def _create_tree(path):
    dirname = op.dirname(path)
    if not op.isdir(dirname):
        os.makedirs(dirname)
    with open(path, "w") as writer:
        writer.write("send2trash test")


@pytest.fixture
def testdir(tmp_path):
    dirname = "\\\\?\\" + str(tmp_path)
    assert op.exists(dirname) is True
    yield dirname
    shutil.rmtree(dirname, ignore_errors=True)


@pytest.fixture
def testfile(testdir):
    file = op.join(testdir, "testfile.txt")
    _create_tree(file)
    assert op.exists(file) is True
    yield file
    # Note dir will cleanup the file


@pytest.fixture
def testfiles(testdir):
    files = [op.join(testdir, "testfile{}.txt".format(index)) for index in range(10)]
    [_create_tree(file) for file in files]
    assert all([op.exists(file) for file in files]) is True
    yield files
    # Note dir will cleanup the files


def _trash_folder(dir, fcn):
    fcn(dir)
    assert op.exists(dir) is False


def _trash_file(file, fcn):
    fcn(file)
    assert op.exists(file) is False


def _trash_multifile(files, fcn):
    fcn(files)
    assert any([op.exists(file) for file in files]) is False


def _file_not_found(dir, fcn):
    file = op.join(dir, "otherfile.txt")
    pytest.raises(OSError, fcn, file)


def _multi_byte_unicode(dir, fcn):
    single_file = op.join(dir, "😇.txt")
    _create_tree(single_file)
    assert op.exists(single_file) is True
    fcn(single_file)
    assert op.exists(single_file) is False
    files = [op.join(dir, "😇{}.txt".format(index)) for index in range(10)]
    [_create_tree(file) for file in files]
    assert all([op.exists(file) for file in files]) is True
    fcn(files)
    assert any([op.exists(file) for file in files]) is False


def test_trash_folder(testdir):
    _trash_folder(testdir, s2t)


def test_trash_file(testfile):
    _trash_file(testfile, s2t)


def test_trash_multifile(testfiles):
    _trash_multifile(testfiles, s2t)


def test_file_not_found(testdir):
    _file_not_found(testdir, s2t)


def test_trash_folder_modern(testdir):
    _trash_folder(testdir, s2t_modern)


def test_trash_file_modern(testfile):
    _trash_file(testfile, s2t_modern)


def test_trash_multifile_modern(testfiles):
    _trash_multifile(testfiles, s2t_modern)


def test_file_not_found_modern(testdir):
    _file_not_found(testdir, s2t_modern)


def test_multi_byte_unicode_modern(testdir):
    _multi_byte_unicode(testdir, s2t_modern)


def test_trash_folder_legacy(testdir):
    _trash_folder(testdir, s2t_legacy)


def test_trash_file_legacy(testfile):
    _trash_file(testfile, s2t_legacy)


def test_trash_multifile_legacy(testfiles):
    _trash_multifile(testfiles, s2t_legacy)


def test_file_not_found_legacy(testdir):
    _file_not_found(testdir, s2t_legacy)


def test_multi_byte_unicode_legacy(testdir):
    _multi_byte_unicode(testdir, s2t_legacy)


# Long path tests
@pytest.fixture
def longdir(tmp_path):
    dirname = "\\\\?\\" + str(tmp_path)
    name = "A" * 100
    yield op.join(dirname, name, name, name)
    try:
        shutil.rmtree(dirname, ignore_errors=True)
    except TypeError:
        pass


@pytest.fixture
def longfile(longdir):
    name = "A" * 100
    path = op.join(longdir, name + "{}.txt")
    file = path.format("")
    _create_tree(file)
    assert op.exists(file) is True
    yield file


@pytest.fixture
def longfiles(longdir):
    name = "A" * 100
    path = op.join(longdir, name + "{}.txt")
    files = [path.format(index) for index in range(10)]
    [_create_tree(file) for file in files]
    assert all([op.exists(file) for file in files]) is True
    yield files


# NOTE: both legacy and modern test "pass" on windows, however sometimes with the same path
# they do not actually recycle files but delete them.  Noticed this when testing with the
# recycle bin open, noticed later tests actually worked, modern version can actually detect
# when this happens but not stop it at this moment, and we need a way to verify it when testing.
def test_trash_long_file_modern(longfile):
    _trash_file(longfile, s2t_modern)


def test_trash_long_multifile_modern(longfiles):
    _trash_multifile(longfiles, s2t_modern)


#     @pytest.skipif(
#         op.splitdrive(os.getcwd())[0] != op.splitdrive(gettempdir())[0],
#         "Cannot trash long path from other drive",
#     )
#     def test_trash_long_folder_modern(self):
#         self._trash_folder(s2t_modern)


def test_trash_long_file_legacy(longfile):
    _trash_file(longfile, s2t_legacy)


def test_trash_long_multifile_legacy(longfiles):
    _trash_multifile(longfiles, s2t_legacy)


#     @pytest.skipif(
#         op.splitdrive(os.getcwd())[0] != op.splitdrive(gettempdir())[0],
#         "Cannot trash long path from other drive",
#     )
#     def test_trash_long_folder_legacy(self):
#         self._trash_folder(s2t_legacy)


def test_trash_nothing_legacy():
    try:
        s2t_legacy([])
    except Exception as ex:
        assert False, "Exception thrown when trashing nothing: {}".format(ex)


def test_trash_nothing_modern():
    try:
        s2t_modern([])
    except Exception as ex:
        assert False, "Exception thrown when trashing nothing: {}".format(ex)



# encoding: utf-8
import pytest
import codecs
import os
import sys
from os import path as op
from send2trash.compat import PY3
from send2trash import TrashPermissionError

try:
    from configparser import ConfigParser
except ImportError:
    # py2
    from ConfigParser import ConfigParser  # noqa: F401

from tempfile import mkdtemp, NamedTemporaryFile
import shutil
import stat
import uuid

if sys.platform != "win32":
    import send2trash.plat_other
    from send2trash.plat_other import send2trash as s2t

    INFO_SUFFIX = send2trash.plat_other.INFO_SUFFIX.decode()
    HOMETRASH = send2trash.plat_other.HOMETRASH
else:
    pytest.skip("Skipping non-windows tests", allow_module_level=True)


@pytest.fixture
def testfile():
    file = NamedTemporaryFile(dir=op.expanduser("~"), prefix="send2trash_test", delete=False)
    file.close()
    assert op.exists(file.name) is True
    yield file
    # Cleanup trash files on supported platforms
    if sys.platform != "win32":
        name = op.basename(file.name)
        # Remove trash files if they exist
        if op.exists(op.join(HOMETRASH, "files", name)):
            os.remove(op.join(HOMETRASH, "files", name))
            os.remove(op.join(HOMETRASH, "info", name + INFO_SUFFIX))
    if op.exists(file.name):
        os.remove(file.name)


@pytest.fixture
def testfiles():
    files = list(
        map(
            lambda index: NamedTemporaryFile(
                dir=op.expanduser("~"),
                prefix="send2trash_test{}".format(index),
                delete=False,
            ),
            range(10),
        )
    )
    [file.close() for file in files]
    assert all([op.exists(file.name) for file in files]) is True
    yield files
    filenames = [op.basename(file.name) for file in files]
    [os.remove(op.join(HOMETRASH, "files", filename)) for filename in filenames]
    [os.remove(op.join(HOMETRASH, "info", filename + INFO_SUFFIX)) for filename in filenames]


def test_trash(testfile):
    s2t(testfile.name)
    assert op.exists(testfile.name) is False


def test_multitrash(testfiles):
    filenames = [file.name for file in testfiles]
    s2t(filenames)
    assert any([op.exists(filename) for filename in filenames]) is False


def touch(path):
    with open(path, "a"):
        os.utime(path, None)


def _filesys_enc():
    enc = sys.getfilesystemencoding()
    # Get canonical name of codec
    return codecs.lookup(enc).name


@pytest.fixture
def gen_unicode_file():
    name = u"send2trash_tést1"
    file = op.join(op.expanduser(b"~"), name.encode("utf-8"))
    touch(file)
    assert op.exists(file) is True
    yield file
    # Cleanup trash files on supported platforms
    if sys.platform != "win32" and op.exists(op.join(HOMETRASH, "files", name)):
        os.remove(op.join(HOMETRASH, "files", name))
        os.remove(op.join(HOMETRASH, "info", name + INFO_SUFFIX))
    if op.exists(file):
        os.remove(file)


@pytest.mark.skipif(_filesys_enc() == "ascii", reason="Requires Unicode filesystem")
def test_trash_bytes(gen_unicode_file):
    s2t(gen_unicode_file)
    assert not op.exists(gen_unicode_file)


@pytest.mark.skipif(_filesys_enc() == "ascii", reason="Requires Unicode filesystem")
def test_trash_unicode(gen_unicode_file):
    s2t(gen_unicode_file.decode(sys.getfilesystemencoding()))
    assert not op.exists(gen_unicode_file)


class ExtVol:
    def __init__(self, path):
        self.trash_topdir = path
        if PY3:
            self.trash_topdir_b = os.fsencode(self.trash_topdir)
        else:
            self.trash_topdir_b = self.trash_topdir

        def s_getdev(path):
            from send2trash.plat_other import is_parent

            st = os.lstat(path)
            if is_parent(self.trash_topdir, path):
                return "dev"
            return st.st_dev

        def s_ismount(path):
            if op.realpath(path) in (
                op.realpath(self.trash_topdir),
                op.realpath(self.trash_topdir_b),
            ):
                return True
            return old_ismount(path)

        self.old_ismount = old_ismount = op.ismount
        self.old_getdev = send2trash.plat_other.get_dev
        send2trash.plat_other.os.path.ismount = s_ismount
        send2trash.plat_other.get_dev = s_getdev

    def cleanup(self):
        send2trash.plat_other.get_dev = self.old_getdev
        send2trash.plat_other.os.path.ismount = self.old_ismount
        shutil.rmtree(self.trash_topdir)


@pytest.fixture
def gen_ext_vol():
    trash_topdir = mkdtemp(prefix="s2t")
    volume = ExtVol(trash_topdir)
    file_name = "test.txt"
    file_path = op.join(volume.trash_topdir, file_name)
    touch(file_path)
    assert op.exists(file_path) is True
    yield volume, file_name, file_path
    volume.cleanup()


def test_trash_topdir(gen_ext_vol):
    trash_dir = op.join(gen_ext_vol[0].trash_topdir, ".Trash")
    os.mkdir(trash_dir, 0o777 | stat.S_ISVTX)

    s2t(gen_ext_vol[2])
    assert op.exists(gen_ext_vol[2]) is False
    assert op.exists(op.join(trash_dir, str(os.getuid()), "files", gen_ext_vol[1])) is True
    assert (
        op.exists(
            op.join(
                trash_dir,
                str(os.getuid()),
                "info",
                gen_ext_vol[1] + INFO_SUFFIX,
            )
        )
        is True
    )
    # info relative path (if another test is added, with the same fileName/Path,
    # then it gets renamed etc.)
    cfg = ConfigParser()
    cfg.read(op.join(trash_dir, str(os.getuid()), "info", gen_ext_vol[1] + INFO_SUFFIX))
    assert (gen_ext_vol[1] == cfg.get("Trash Info", "Path", raw=True)) is True


def test_trash_topdir_fallback(gen_ext_vol):
    s2t(gen_ext_vol[2])
    assert op.exists(gen_ext_vol[2]) is False
    assert (
        op.exists(
            op.join(
                gen_ext_vol[0].trash_topdir,
                ".Trash-" + str(os.getuid()),
                "files",
                gen_ext_vol[1],
            )
        )
        is True
    )


def test_trash_topdir_failure(gen_ext_vol):
    os.chmod(gen_ext_vol[0].trash_topdir, 0o500)  # not writable to induce the exception
    pytest.raises(TrashPermissionError, s2t, [gen_ext_vol[2]])
    os.chmod(gen_ext_vol[0].trash_topdir, 0o700)  # writable to allow deletion


def test_trash_symlink(gen_ext_vol):
    # Generating a random uuid named path for symlink
    sl_dir = op.join(op.expanduser("~"), "s2t_" + str(uuid.uuid4()))
    os.mkdir(op.join(gen_ext_vol[0].trash_topdir, "subdir"), 0o700)
    file_path = op.join(gen_ext_vol[0].trash_topdir, "subdir", gen_ext_vol[1])
    touch(file_path)
    os.symlink(op.join(gen_ext_vol[0].trash_topdir, "subdir"), sl_dir)
    s2t(op.join(sl_dir, gen_ext_vol[1]))
    assert op.exists(file_path) is False
    assert (
        op.exists(
            op.join(
                gen_ext_vol[0].trash_topdir,
                ".Trash-" + str(os.getuid()),
                "files",
                gen_ext_vol[1],
            )
        )
        is True
    )
    os.remove(sl_dir)



# encoding: utf-8
import os
import sys
import pytest
from tempfile import NamedTemporaryFile
from os import path as op

from send2trash.__main__ import main as trash_main

# Only import HOMETRASH on supported platforms
if sys.platform != "win32":
    from send2trash.plat_other import HOMETRASH


@pytest.fixture
def file():
    file = NamedTemporaryFile(dir=op.expanduser("~"), prefix="send2trash_test", delete=False)
    file.close()
    # Verify file was actually created
    assert op.exists(file.name) is True
    yield file.name
    # Cleanup trash files on supported platforms
    if sys.platform != "win32":
        name = op.basename(file.name)
        # Remove trash files if they exist
        if op.exists(op.join(HOMETRASH, "files", name)):
            os.remove(op.join(HOMETRASH, "files", name))
            os.remove(op.join(HOMETRASH, "info", name + ".trashinfo"))
    if op.exists(file.name):
        os.remove(file.name)


def test_trash(file):
    trash_main(["-v", file])
    assert op.exists(file) is False


def test_no_args(file):
    pytest.raises(SystemExit, trash_main, [])
    pytest.raises(SystemExit, trash_main, ["-v"])
    assert op.exists(file) is True
